<?php
// Prevent direct file access
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles display and processing of the password form used for password protected categories and products.
 *
 * @package   WooCommerce_Protected_Categories
 * @author    Barn2 Media <info@barn2.co.uk>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class WC_PPC_Password_Form {

	private static $form_id = 1;

	public static function register_login_handler() {
		add_action( 'wp', array( __CLASS__, 'handle_login' ) );
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'register_scripts' ) );
	}

	// Handle submission of category login form.
	public static function handle_login() {
		global $wp_did_header;

		// Bail if we're not loading a front-end template, or password form not submitted.
		if ( ! $wp_did_header || 'POST' !== $_SERVER['REQUEST_METHOD'] || ! filter_input( INPUT_POST, 'wc_ppc_login', FILTER_VALIDATE_INT ) ) {
			return;
		}

		$password = filter_input( INPUT_POST, 'post_password', FILTER_SANITIZE_STRING );
		if ( ! $password ) {
			return;
		}

		$categories = array();

		if ( is_product_category() ) {
			// Form submitted from a product catetory.
			$categories = array( WC_PPC_Util::get_category_visibility( get_queried_object_id() ) );
		} elseif ( is_product() ) {
			// Form submitted from a single product.
			$categories = WC_PPC_Util::get_the_category_visibility();
		} else {
			// Form submitted from central login page, shortcode or widget, so we fetch all categories.
			$categories = WC_PPC_Util::to_category_visibilities( WC_PPC_Util::get_product_categories() );
		}

		if ( ! $categories ) {
			return;
		}

		// Check the password against each category.
		foreach ( $categories as $category ) {

			if ( $term_id = $category->check_password( $password ) ) {
				// Got a valid password, so set password cookie then redirect.
				WC_PPC_Util::set_password_cookie( $term_id, $password );

				if ( is_product_category() || is_product() ) {
					wp_safe_redirect( add_query_arg( null, null ) );
				} else {
					wp_safe_redirect( get_term_link( $term_id, 'product_cat' ) );
				}
				return;
			}
		}
	}

	public static function register_scripts() {
		$suffix = WC_PPC_Util::get_script_suffix();
		wp_register_style( 'wc-ppc', plugins_url( "assets/css/wc-ppc{$suffix}.css", WC_Protected_Categories_Plugin::FILE ), array(), WC_Protected_Categories_Plugin::VERSION );
	}

	public static function display_password_form( $args = array() ) {
		echo self::get_password_form( $args );
	}

	public static function get_password_form_heading() {
		return apply_filters( 'wc_ppc_password_form_heading', get_option( 'wc_ppc_login_title', __( 'Please Login', 'wc-cat-protect' ) ) );
	}

	public static function get_password_form( $args = array() ) {

		wp_enqueue_style( 'wc-ppc' );

		$args = wp_parse_args( $args, array(
			'message'			 => get_option( 'wc_ppc_password_form' ),
			'heading'			 => self::get_password_form_heading(),
			'show_heading'		 => true,
			'label'				 => get_option( 'wc_ppc_form_password_label', __( 'Password:', 'wc-cat-protect' ) ),
			'button_text'		 => get_option( 'wc_ppc_form_button', _x( 'Login', 'category password form', 'wc-cat-protect' ) ),
			'container_class'	 => get_option( 'wc_ppc_form_container_class' ),
			) );

		$container_class = trim( 'wc-ppc-form-wrapper ' . $args['container_class'] );
		$form_action	 = add_query_arg( null, null );

		$heading = $args['show_heading'] ? sprintf( '<h3 class="category-login__heading">%s</h3>', $args['heading'] ) : '';
		$message = $args['message'] ? do_shortcode( wpautop( $args['message'] ) ) : '';
		$message .= self::get_error_message( self::$form_id );

		$label_id	 = 'pwbox-' . self::$form_id;
		$label_text	 = $args['label'];
		$placeholder = '';

		if ( 'yes' === get_option( 'wc_ppc_form_label_placeholder' ) ) {
			$placeholder = ' placeholder="' . esc_attr( $label_text ) . '"';
			$label_text	 = '';
		} elseif ( $label_text ) {
			// If we have a password label, add a space after to align the form better
			$label_text .= ' ';
		}

		ob_start();
		do_action( 'wc_ppc_before_password_form' );
		?>
		<div class="<?php echo esc_attr( $container_class ); ?>">
			<form action="<?php echo esc_url( $form_action ); ?>" class="category-login wc-ppc-password-form post-password-form" method="post">
				<?php echo $heading; ?>
				<div class="category-login__text"><?php echo $message; ?></div>
				<p class="category-login__fields wc-ppc-password-form-inputs">
					<label class="category-login__label wc-ppc-password-label"><?php echo esc_html( $label_text ); ?><input name="post_password" id="<?php echo esc_attr( $label_id ); ?>" class="category-login__password wc-ppc-password" type="password" size="25"<?php echo $placeholder; ?> /></label>
					<input class="category-login__submit" type="submit" name="Submit" value="<?php echo esc_attr( $args['button_text'] ); ?>" />
					<input type="hidden" name="wc_ppc_login" value="<?php echo esc_attr( self::$form_id ); ?>" />
				</p>
			</form>
		</div>
		<?php
		do_action( 'wc_ppc_after_password_form' );

		self::$form_id ++;

		return apply_filters( 'wc_ppc_password_form', ob_get_clean() );
	}

	private static function get_error_message() {

		if ( 'POST' !== $_SERVER['REQUEST_METHOD'] ) {
			return '';
		}

		$submitted_form_id = filter_input( INPUT_POST, 'wc_ppc_login', FILTER_VALIDATE_INT );

		// Check the passed form ID matches the one in $_POST
		if ( ! $submitted_form_id || self::$form_id !== (int) $submitted_form_id ) {
			return '';
		}

		$error		 = '';
		$password	 = filter_input( INPUT_POST, 'post_password', FILTER_SANITIZE_STRING );

		if ( ! $password ) {
			$error = __( 'Please enter a password.', 'wc-cat-protect' );
		} else {
			// If we have a password, and we're doing a POST request (therefore we haven't redirected), then the password must be wrong.
			$error = __( 'Incorrect password, please try again.', 'wc-cat-protect' );
		}

		if ( $error ) {
			return sprintf( '<p class="category-login__error wc-ppc-login-error">%s</p>', $error );
		}
		return '';
	}

}
// class WC_PPC_Password_Form