<?php
// Prevent direct file access
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles the protection of product categories and products by overriding the template or main
 * query for the current page.
 *
 * @package   WooCommerce_Protected_Categories
 * @author    Barn2 Media <info@barn2.co.uk>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class WC_PPC_Template_Handler {

	public static function register_template_hooks() {
		// Run on 'wp' hook so we can set cache constants.
		add_action( 'wp', array( __CLASS__, 'protect_shop_pages' ), 20 ); // after WC_PPC_Password_Form:handle_login
	}

	public static function protect_shop_pages() {
		global $wp_did_header;

		// Return if we're not loading a template on the front-end.
		if ( ! $wp_did_header ) {
			return;
		}

		$is_protected = false;

		if ( is_product_category() ) {
			$category		 = WC_PPC_Util::get_category_visibility( get_queried_object_id() );
			$is_protected	 = self::handle_category_protection( array( $category ) );
		} elseif ( is_product() ) {
			$is_protected = self::handle_category_protection( WC_PPC_Util::get_the_category_visibility() );
		} elseif ( is_shop() ) {
			$shop_page = wc_get_page_id( 'shop' );

			// If central login page is the main shop page, hide all products/categories (the shortcode will handle display of form).
			if ( $shop_page > 0 && $shop_page === absint( get_option( 'wc_ppc_category_login_page' ) ) ) {
				$is_protected	 = true;
				$loop_buffer	 = new WC_PPC_Loop_Buffer( false );

				add_action( 'woocommerce_before_shop_loop', array( $loop_buffer, 'start_buffer' ), 1 ); // before ordering & results count
				add_action( 'woocommerce_after_shop_loop', array( $loop_buffer, 'end_buffer' ), 100 ); // after pagination

				add_action( 'woocommerce_before_template_part', array( $loop_buffer, 'start_buffer_no_products' ), 1 );
				add_action( 'woocommerce_after_template_part', array( $loop_buffer, 'end_buffer_no_products' ), 100 );

				do_action( 'wc_ppc_handle_protected_shop_page' );
			}
		}

		if ( $is_protected ) {
			do_action( 'wc_ppc_protected_woocommerce_page' );

			self::prevent_indexing();
			self::prevent_caching();
		}
	}

	private static function handle_category_protection( $categories ) {
		if ( ! $categories ) {
			return false;
		}

		// If any of the categories is unlocked, then we don't protect.
		if ( array_filter( $categories, array( __CLASS__, 'is_unlocked' ) ) ) {
			return false;
		}

		$lowest_pp_level = false;
		$lowest_up_level = false;

		// Append ancestors for each category, and loop through each to check for protection.
		foreach ( array_map( array( __CLASS__, 'append_ancestors' ), $categories ) as $cat_with_ancestors ) {
			$password_protected_keys = array_keys( array_filter( $cat_with_ancestors, array( __CLASS__, 'is_password_protected' ) ) );
			$user_protected_keys	 = array_keys( array_filter( $cat_with_ancestors, array( __CLASS__, 'is_user_protected' ) ) );

			// At least one password protected category in hierarchy, so work out the lowest level it applies (lower index means lower down the hierarchy).
			if ( $password_protected_keys ) {
				$pp_level		 = reset( $password_protected_keys );
				$lowest_pp_level = ( false !== $lowest_pp_level ) ? min( $pp_level, $lowest_pp_level ) : $pp_level;
			}
			// At least one user, role or private category in hierarchy, so work out the lowest level it applies (lower index mean lower down the hierarchy).
			if ( $user_protected_keys ) {
				$up_level		 = reset( $user_protected_keys );
				$lowest_up_level = ( false !== $lowest_up_level ) ? min( $up_level, $lowest_up_level ) : $up_level;
			}
		}

		// If no password or user protection, then categories are not protected.
		if ( false === $lowest_pp_level && false === $lowest_up_level ) {
			return false;
		}

		if ( false === $lowest_up_level || ( is_int( $lowest_pp_level ) && $lowest_pp_level <= $lowest_up_level ) ) {
			// If no user protection, or password is lower or same level in hierarchy, then we password protect.
			self::handle_password_protection();
			return true;
		} elseif ( false === $lowest_pp_level || ( is_int( $lowest_up_level ) && $lowest_up_level < $lowest_pp_level ) ) {
			// If no password protection, or user protection lower down the hierarchy, then we user protect.
			self::handle_user_protection();
			return true;
		}

		return false;
	}

	private static function append_ancestors( $category ) {
		$with_ancestors = $category->ancestors();
		array_unshift( $with_ancestors, $category );
		return $with_ancestors;
	}

	private static function is_unlocked( $category ) {
		return $category->is_unlocked();
	}

	private static function is_password_protected( $category ) {
		return $category->is_password_protected();
	}

	private static function is_user_protected( $category ) {
		return $category->is_user_protected() || $category->is_role_protected() || $category->is_private();
	}

	private static function handle_password_protection() {
		global $wp, $wp_query;

		$post_id				 = rand( 1000000, 10000000 ); // attempt to avoid clash with a valid post
		$post					 = new stdClass();
		$post->ID				 = $post_id;
		$post->post_author		 = 1;
		$post->post_date		 = current_time( 'mysql' );
		$post->post_date_gmt	 = current_time( 'mysql', 1 );
		$post->post_status		 = 'publish';
		$post->comment_status	 = 'closed';
		$post->comment_count	 = 0;
		$post->ping_status		 = 'closed';
		$post->post_type		 = 'page';
		$post->filter			 = 'raw'; // important
		$post->post_name		 = 'category-login-' . $post_id; // append post ID to avoid clash
		$post->post_title		 = WC_PPC_Password_Form::get_password_form_heading();
		$post->post_content		 = '[' . WC_PPC_Login_Shortcode::SHORTCODE . ' show_heading="false"]';

		// Convert to WP_Post object
		$wp_post = new WP_Post( $post );

		// Add our fake post to the cache
		wp_cache_add( $post_id, $wp_post, 'posts' );
		wp_cache_add( $post_id, array( true ), 'post_meta' );

		// Override main query
		$wp_query->post					 = $wp_post;
		$wp_query->posts				 = array( $wp_post );
		$wp_query->queried_object		 = $wp_post;
		$wp_query->queried_object_id	 = $wp_post->ID;
		$wp_query->found_posts			 = 1;
		$wp_query->post_count			 = 1;
		$wp_query->max_num_pages		 = 1;
		$wp_query->comment_count		 = 0;
		$wp_query->comments				 = array();
		$wp_query->is_singular			 = true;
		$wp_query->is_page				 = true;
		$wp_query->is_single			 = false;
		$wp_query->is_attachment		 = false;
		$wp_query->is_archive			 = false;
		$wp_query->is_category			 = false;
		$wp_query->is_tag				 = false;
		$wp_query->is_tax				 = false;
		$wp_query->is_author			 = false;
		$wp_query->is_date				 = false;
		$wp_query->is_year				 = false;
		$wp_query->is_month				 = false;
		$wp_query->is_day				 = false;
		$wp_query->is_time				 = false;
		$wp_query->is_search			 = false;
		$wp_query->is_feed				 = false;
		$wp_query->is_comment_feed		 = false;
		$wp_query->is_trackback			 = false;
		$wp_query->is_home				 = false;
		$wp_query->is_embed				 = false;
		$wp_query->is_404				 = false;
		$wp_query->is_paged				 = false;
		$wp_query->is_admin				 = false;
		$wp_query->is_preview			 = false;
		$wp_query->is_robots			 = false;
		$wp_query->is_posts_page		 = false;
		$wp_query->is_post_type_archive	 = false;

		// Update globals
		$GLOBALS['wp_query'] = $wp_query;
		$wp->register_globals();

		do_action( 'wc_ppc_handle_password_protection' );
	}

	private static function handle_user_protection() {

		do_action( 'wc_ppc_handle_user_protection' );

		$user_protected_option = get_option( 'wc_ppc_user_protected', '404' );

		if ( '404' === $user_protected_option ) {
			self::handle_404();
		} elseif ( 'wplogin' === $user_protected_option ) {
			wp_safe_redirect( self::get_login_url_with_current_page_redirect() );
		} elseif ( 'page' === $user_protected_option ) {
			if ( $page_id = get_option( 'wc_ppc_user_protected_redirect' ) ) {
				$redirect = add_query_arg( 'redirect_to', urlencode( add_query_arg( null, null ) ), get_permalink( $page_id ) );
				wp_safe_redirect( $redirect );
			} else {
				wp_safe_redirect( self::get_login_url_with_current_page_redirect() );
			}
		}
	}

	private static function get_login_url_with_current_page_redirect() {
		return apply_filters( 'wc_ppc_user_protected_login_url', wp_login_url( add_query_arg( null, null ) ) );
	}

	private static function handle_404() {
		global $wp, $wp_query;

		$wp->handle_404();
		$wp_query->is_404		 = true;
		$wp_query->is_single	 = false;
		$wp_query->is_singular	 = false;
		$wp_query->is_tax		 = false;
		$wp_query->is_archive	 = false;

		/* deprecated 2.0 - renamed 'wc_ppc_handle_404' */
		do_action( 'wc_ppc_private_category_handle_404' );

		do_action( 'wc_ppc_handle_404' );
	}

	private static function prevent_indexing() {
		@header( 'X-Robots-Tag: noindex, nofollow' );
		add_action( 'wp_head', array( __CLASS__, 'meta_robots_noindex_head' ), 5 );

		do_action( 'wc_ppc_prevent_indexing' );
	}

	public static function meta_robots_noindex_head() {
		echo '<meta name="robots" content="noindex, nofollow" />' . "\n";
	}

	private static function prevent_caching() {
		// Set headers to prevent caching
		nocache_headers();

		// Set constants to prevent caching in certain caching plugins
		if ( ! defined( 'DONOTCACHEPAGE' ) ) {
			define( 'DONOTCACHEPAGE', true );
		}
		if ( ! defined( 'DONOTCACHEOBJECT' ) ) {
			define( 'DONOTCACHEOBJECT', true );
		}
		if ( ! defined( 'DONOTCACHEDB' ) ) {
			define( 'DONOTCACHEDB', true );
		}

		do_action( 'wc_ppc_prevent_caching' );
	}

}
// class WC_PPC_Template_Handler